﻿using System;
using System.Windows;
using Navigation_Pages_MVVM.View;
using Navigation_Pages_MVVM.ViewModel;
using Navigation_Pages_MVVM.Singletons;
using System.Reflection;
using System.IO;

namespace Navigation_Pages_MVVM
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        protected override void OnStartup(StartupEventArgs e)
        {
            base.OnStartup(e);

            // Allow for arguments
            ParseArgs(e.Args);

            // Load and show the MainWindow
            MainWindowViewModel vm = new MainWindowViewModel();
            MainWindow main = new MainWindow();
            Navigator.NavigationService = main.NavigationFrame.NavigationService;
            main.DataContext = vm;
            main.Show();

            // If they pass in a /? 
            String showParameters = string.Empty;
            if (DataStore.Information.TryGetValue("?", out showParameters)
                && showParameters.Equals("True", StringComparison.CurrentCultureIgnoreCase))
            {
                // Load and navigate to the first page
                ArgumentsPageViewModel argvm = new ArgumentsPageViewModel();
                ArgumentsPage ap = new ArgumentsPage();
                ap.DataContext = argvm;
                Navigator.NavigationService.Navigate(ap);
            }
            else
            {
                // Load and navigate to the first page
                Page1ViewModel pagevm = new Page1ViewModel();
                Page1 p1 = new Page1();
                p1.DataContext = pagevm;
                Navigator.NavigationService.Navigate(p1);
            }
        }

        /// <summary>
        /// This function accepts all arguments and puts them in the DataStore.
        /// Only valid arguments are used though.
        /// </summary>
        /// <param name="inArgs"></param>
        private void ParseArgs(String[] inArgs)
        {
            // Add exe details to DataStore, such as the Setup.exe path and directory
            DataStore.ExePath = Assembly.GetExecutingAssembly().Location;
            DataStore.ExeFile = Path.GetFileName(DataStore.ExePath);
            DataStore.ExeDir = Path.GetDirectoryName(DataStore.ExePath);

            // Parse passed in args
            foreach (string arg in inArgs)
            {
                // We don't care what args they pass in, we will store all the 
                // information in the DataStore.
                // 
                // Parameters should be in one of the following formats:
                //    /param1
                //    param1
                //    /Param1=Value
                //    Param1=Value
                // 

                int Parameter = 0;
                int Value = 1;

                string[] data = new String[2];
                string[] tmp = arg.Split("=".ToCharArray(), 2, StringSplitOptions.None);
                data[Parameter] = tmp[Parameter];

                if (tmp.Length > 1)
                    data[Value] = tmp[Value];

                // Remove the slash if there is one
                while (data[Parameter].StartsWith("/") || data[Parameter].StartsWith("-"))
                    data[Parameter] = data[Parameter].Substring(1);

                // If the KeyValue doesn't exist set it to true
                if (String.IsNullOrWhiteSpace(data[Value]))
                    data[Value] = "true";
                else
                    // If the KeyValue exists, trim quotes and then trim spaces
                    data[Value] = data[Value].Trim("\"".ToCharArray()).Trim();

                DataStore.Information.Add(data[Parameter], data[Value]);
            }
        }
    }
}
