﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;
using System.Threading;
using System.Resources;
using MVVM;
using Navigation_Pages_MVVM.Singletons;

namespace Navigation_Pages_MVVM.ViewModel
{
    interface IValid
    {
        bool IsValid();
    }

    public abstract class PageViewModelBase : ViewModelBase, IValid
    {
        #region Member Variables
        protected ButtonViewModel _ButtonCancel;
        protected ButtonViewModel _ButtonContinue;
        protected ButtonViewModel _ButtonPrevious;
        #endregion

        #region Constructors
        /// <summary>
        /// The default controller
        /// </summary>
        /// <param name="inProductController"></param>
        public PageViewModelBase()
        {
        }
        #endregion

        #region Properties
        public String Heading { get; set; }

        /// <summary>
        /// This is a ViewModel for the Button
        /// </summary>
        public virtual ButtonViewModel ButtonCancel
        {
            get
            {
                if (_ButtonCancel == null)
                {
                    _ButtonCancel = new ButtonViewModel()
                    {
                        Text = "Cancel",
                        IsVisible = true,
                        IsEnabled = true,
                        ButtonCommand = new RelayCommand(param => ButtonCancelClick())
                    };
                }
                return _ButtonCancel;
            }
            set
            {
                _ButtonCancel = value;
                NotifyPropertyChanged("ButtonCancel");
            }
        }

        public virtual ButtonViewModel ButtonContinue
        {
            get
            {
                if (_ButtonContinue == null)
                {
                    _ButtonContinue = new ButtonViewModel()
                    {
                        Text = "Next",
                        IsVisible = true,
                        IsEnabled = true, // While this is set to true, the delegate can make it false
                        ButtonCommand = new RelayCommand(param => ButtonContinueClick()),
                        IsEnabledMethod = IsValid
                    };
                }
                return _ButtonContinue;
            }
            set
            {
                _ButtonContinue = value;
                NotifyPropertyChanged("ButtonContinue");
            }
        }

        public virtual ButtonViewModel ButtonPrevious
        {
            get
            {
                if (_ButtonPrevious == null)
                {
                    _ButtonPrevious = new ButtonViewModel()
                    {
                        Text = "Back",
                        IsVisible = true,
                        IsEnabled = true,
                        ButtonCommand = new RelayCommand(param => ButtonPreviousClick())
                    };
                }
                return _ButtonPrevious; ;
            }
            set
            {
                _ButtonPrevious = value;
                NotifyPropertyChanged("ButtonPrevious");
            }
        }

        /// <summary>
        /// Each page should do validations and only enable the ButtonNext when the page is valid.
        /// </summary>
        public abstract bool IsValid();
        #endregion

        #region Functions
        /// <summary>
        /// This function is called when the Previous Button is clicked.
        /// </summary>
        public virtual void ButtonPreviousClick()
        {
            Navigator.NavigationService.GoBack();
        }

        /// <summary>
        /// This function is called when the Cancel Button is clicked.
        /// </summary>
        public virtual void ButtonCancelClick()
        {
            Navigator.Cancel();
        }

        /// <summary>
        /// This function is called when the Continue Button is clicked.
        /// </summary>
        public abstract void ButtonContinueClick();
        #endregion
    }
}
